"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCategoriesCrud = void 0;
var _handlerGraphql = require("@webiny/handler-graphql");
var _error = _interopRequireDefault(require("@webiny/error"));
var _pubsub = require("@webiny/pubsub");
var _validation = require("./categories/validation");
var _utils = require("@webiny/utils");
const createCategoriesCrud = params => {
  const {
    context,
    storageOperations,
    categoriesPermissions,
    pagesPermissions,
    getLocaleCode,
    getTenantId
  } = params;
  const onCategoryBeforeCreate = (0, _pubsub.createTopic)("pageBuilder.onCategoryBeforeCreate");
  const onCategoryAfterCreate = (0, _pubsub.createTopic)("pageBuilder.onCategoryAfterCreate");
  const onCategoryBeforeUpdate = (0, _pubsub.createTopic)("pageBuilder.onCategoryBeforeUpdate");
  const onCategoryAfterUpdate = (0, _pubsub.createTopic)("pageBuilder.onCategoryAfterUpdate");
  const onCategoryBeforeDelete = (0, _pubsub.createTopic)("pageBuilder.onCategoryBeforeDelete");
  const onCategoryAfterDelete = (0, _pubsub.createTopic)("pageBuilder.onCategoryAfterDelete");
  return {
    /**
     * Lifecycle events - deprecated in 5.34.0 - will be removed from 5.36.0
     */
    onBeforeCategoryCreate: onCategoryBeforeCreate,
    onAfterCategoryCreate: onCategoryAfterCreate,
    onBeforeCategoryUpdate: onCategoryBeforeUpdate,
    onAfterCategoryUpdate: onCategoryAfterUpdate,
    onBeforeCategoryDelete: onCategoryBeforeDelete,
    onAfterCategoryDelete: onCategoryAfterDelete,
    /**
     * Introduced in 5.34.0
     */
    onCategoryBeforeCreate,
    onCategoryAfterCreate,
    onCategoryBeforeUpdate,
    onCategoryAfterUpdate,
    onCategoryBeforeDelete,
    onCategoryAfterDelete,
    /**
     * This method should return category or null. No error throwing on not found.
     */
    async getCategory(slug, options = {
      auth: true
    }) {
      const {
        auth
      } = options;
      const params = {
        where: {
          slug,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      };
      if (auth === false) {
        return await storageOperations.categories.get(params);
      }
      try {
        await categoriesPermissions.ensure({
          rwd: "r"
        });
      } catch {
        // If we don't have the necessary `categoryPermission` permission, let's still check if the
        // user has the permission to write pages. If so, we still want to allow listing categories,
        // because this is needed in order to create a page.
        await pagesPermissions.ensure({
          rwd: "w"
        });
      }
      let category = null;
      try {
        category = await storageOperations.categories.get(params);
      } catch (ex) {
        throw new _error.default(ex.message || "Could not load category by slug.", ex.code || "GET_CATEGORY_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
      if (!category) {
        return null;
      }
      await categoriesPermissions.ensure({
        owns: category.createdBy
      });
      return category;
    },
    async listCategories() {
      try {
        await categoriesPermissions.ensure({
          rwd: "r"
        });
      } catch {
        // If we don't have the necessary `categoryPermission` permission, let's still check if the
        // user has the permission to write pages. If so, we still want to allow listing categories,
        // because this is needed in order to create a page.
        await pagesPermissions.ensure({
          rwd: "w"
        });
      }
      const params = {
        where: {
          tenant: getTenantId(),
          locale: getLocaleCode()
        },
        sort: ["createdOn_ASC"]
      };
      // If user can only manage own records, add the createdBy to where values.
      if (await categoriesPermissions.canAccessOnlyOwnRecords()) {
        const identity = context.security.getIdentity();
        params.where.createdBy = identity.id;
      }
      try {
        const [items] = await storageOperations.categories.list(params);
        return items;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list categories by given params.", ex.code || "LIST_CATEGORY_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
    },
    async createCategory(input) {
      await categoriesPermissions.ensure({
        rwd: "w"
      });
      const validationResult = await (0, _validation.createCategoryCreateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const existingCategory = await this.getCategory(input.slug, {
        auth: false
      });
      if (existingCategory) {
        throw new _handlerGraphql.NotFoundError(`Category with slug "${input.slug}" already exists.`);
      }
      const identity = context.security.getIdentity();
      const data = validationResult.data;
      const category = {
        ...data,
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          type: identity.type,
          displayName: identity.displayName
        },
        tenant: getTenantId(),
        locale: getLocaleCode()
      };
      try {
        await onCategoryBeforeCreate.publish({
          category
        });
        const result = await storageOperations.categories.create({
          input: data,
          category
        });
        await onCategoryAfterCreate.publish({
          category: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create category.", ex.code || "CREATE_CATEGORY_ERROR", {
          ...(ex.data || {}),
          category
        });
      }
    },
    async updateCategory(slug, input) {
      await categoriesPermissions.ensure({
        rwd: "w"
      });
      const original = await this.getCategory(slug);
      if (!original) {
        throw new _handlerGraphql.NotFoundError(`Category "${slug}" not found.`);
      }
      await categoriesPermissions.ensure({
        owns: original.createdBy
      });
      const validationResult = await (0, _validation.createCategoryUpdateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const data = (0, _utils.removeUndefinedValues)(validationResult.data);
      const category = {
        ...original,
        ...data
      };
      try {
        await onCategoryBeforeUpdate.publish({
          original,
          category
        });
        const result = await storageOperations.categories.update({
          input: data,
          original,
          category
        });
        await onCategoryAfterUpdate.publish({
          original,
          category
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update category.", ex.code || "UPDATE_CATEGORY_ERROR", {
          ...(ex.data || {}),
          original,
          category
        });
      }
    },
    async deleteCategory(slug) {
      await categoriesPermissions.ensure({
        rwd: "d"
      });
      const category = await this.getCategory(slug);
      if (!category) {
        throw new _handlerGraphql.NotFoundError(`Category "${slug}" not found.`);
      }
      await categoriesPermissions.ensure({
        owns: category.createdBy
      });

      // Before deleting, let's check if there is a page that's in this category.
      // If so, let's prevent this.
      const [pages] = await this.listLatestPages({
        where: {
          category: category.slug
        },
        limit: 1
      }, {
        auth: false
      });
      if (pages.length > 0) {
        throw new _error.default("Cannot delete category because some pages are linked to it.", "CANNOT_DELETE_CATEGORY_PAGE_EXISTING");
      }
      try {
        await onCategoryBeforeDelete.publish({
          category
        });
        const result = await storageOperations.categories.delete({
          category
        });
        await onCategoryAfterDelete.publish({
          category: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete category.", ex.code || "DELETE_CATEGORY_ERROR", {
          ...(ex.data || {}),
          category
        });
      }
    }
  };
};
exports.createCategoriesCrud = createCategoriesCrud;

//# sourceMappingURL=categories.crud.js.map